



--------------------------------------------------
--- Make entries of namespace available as globals
----
function using(namespace,overwrite)
   if overwrite == nil then overwrite=false end
   if type(namespace)=="table" then
      for name,value in pairs(namespace) do
       if overwrite or _G[name] == nil then 
         _G[name]=value 
       end
      end
   elseif type(namespace)=="string" then 
	 using(_G[namespace])
   else
      error("using: namespace not available")
   end
end



function need(namespace)
 if type(namespace)=="string" then
  if _G[namespace] == nil then 
   error("need("..namespace..")")
  end
 else
   error("need: expect a 'string'")
 end
end


need("mbase")


------------------------------------------------
---
--- General type-check for build-in types and swig types.
---
--- Example:
--- 
--- a = da1Create(10)
--- if swigext_istype(a,"DArray1") then print("is DArray1") end
--- 

function swigext_istype(v1,v2)
  local v,t  -- value to check, type-name
  
  -- detect args
  if type(v1)=="string" then
   t=v1 v=v2
  else
   t=v2 v=v1
  end
  if type(t)~="string" then
   error("expected type-name (string) and value")
  end
  if v == nil then
   return false
  end
  
  -- build-in type
  if type(v)==t then
   return true
  end
  
  -- swig type
  local function to_st(t)
    local st=t
    st = string.gsub(st," ","")
    st = string.gsub(st,"<","T")
    st = string.gsub(st,">","_t")
    return st
  end
  
  local st = swig_type(v)
  local tt = to_st(t)
  
  if st == "_p_"..tt then
   return true
  end
  if st == "_p_PtrT"..tt.."_t" then
   return true
  end
  return false
end

istype=swigext_istype


------------------------------------------------
---
--- General type info for build-in types and swig types.
---
function swigext_type(o)
 if type(o) == "userdata" then
  return swig_type(o)
 end
 return type(o)
end


------------------------------------------------
---
--- operator bool() of smartpointer.
---  (built-in types path through)
---

function swigext_bool(ptr)
 if type(ptr) ~= "userdata" then
  return ptr
 end
 if ptr:__deref__() then
  return true
 end
 return false
end

bool=swigext_bool










function swigext_debug(a1,a2)
 local name,var
 if type(a1) == "string" then
  name = a1
  var = a2
 else
  name = a2
  var = a1
 end
 if name then
  print("type("..name..")=="..swigext_type(var),"val("..name..")=="..tostring(var))
 else
  print("type=="..swigext_type(var),"val=="..tostring(var))
 end
end





------------------------------------------------------------------
--

-- 
-- stdsys: milestone
-- 
function milestoneRead(file)
   _={}
   loadfile(file)()
   print(table.getn(_))
   return _
end


function milestoneRecord()
  local s="return {"..milestoneComment().."}"
  local t=loadstring(s)()
  return  t
end



------------------------------------------------------------------
--

-- 
-- stdsys: args
-- 

-------------------------------------------------------------------
-- split file name into base and extension
function splitfilename(fname)
   i1,i2=string.find(fname,"%.%w*$")
   if not i1 then return fname end
   return string.sub(fname,1,i1-1), string.sub(fname,i1-i2) 
end

-- split file name into dir, base and ext.
-- if one of them is not present then 
-- a nil is returned
-- usage: 
-- dir,base,ext=splitfilenamedir(fname)
-- UNIX only
function splitfilenamedir(fname)
   local dir=nil
   local base=nil
   local ext=nil
   local xfname=fname
   -- check for dir pattern
   local i1,i2=string.find(fname,"%/[^/]*$")
   if i1 then 
      dir=string.sub(fname,1,i1-1)
      xfname=string.sub(fname,i1-i2)
      if xfname==fname then xfname="" end
   end
   -- check for fname.ext
   base=xfname
   i1,i2=string.find(xfname,"%.%w*$")
   if i1 then
      base=string.sub(xfname,1,i1-1)
      ext=string.sub(xfname,i1-i2)
   end
   if base=="" then base=nil end
   return dir,base,ext 
end


-------------------------------------------------------------------
--
-- Process command line arguments.
-- We distinguish three types of command line arguments :
--
-- switches: 
--    These can be used as flags and are collected
--    in the table arg.switches. E.g. if switch -xxx 
--    is present, arg.switches.xxx is set to true
--
-- values: 
--    These are derived from statements like a=b
--    and collected in arg.values.
--    If b is a global variable, it is assigned to arg.values.a.
--    If the string b is a number, it is assigned to arg.values.a.
--    If the string is "true" or "false", it is assigned the corresponding
--             boolean value.
--    Otherwise, the string "b" is assigned to arg.values.a. 
--    
-- files:
--    All arguments which are neither switches, nor assignments
--    are treated as input files. Their existence ist tested.
--    They are collected in the table arg.switches.

function processargs(arg)
   
   -- If argument is a switch (starting with '-') return its name, nil otherwise
   local function is_switch(a)
      if not a then return nil end
      if string.sub(a,1,1) == "-" then return string.sub(a,2) end
      return nil
   end
   
   local eq,s,i
   arg.switches={} -- table to hold values of switches
   arg.files={}    -- table to hold names of files
   arg.values={}   -- table to hold values of assigned arguments

   i=1
   while i<=arg.n   do
      eq=string.find(arg[i],"=")
      sw=is_switch(arg[i])
      s=nil
      if sw then  -- this is a switch
 	 arg.switches[sw]=true  -- add to switch table
      elseif eq then -- this is an assignment
	 local left,right,var
	 left=string.sub(arg[i],1,eq-1) -- extract string left of "="
	 right=string.sub(arg[i],eq+1) -- extract string right of "="

         if  tonumber(right) then     
	    arg.values[left]=tonumber(right) -- assign number 
	 elseif right=="true" then  
	    arg.values[left]=true   -- assign boolean value
	 elseif right=="false" then  
	    arg.values[left]=false  -- assign boolean value
	 else	       
	    arg.values[left]=right  -- assign string
	 end
      else -- assume argument is the name of an existing file
	 -- check if file exists and is readable
	local f=io.open(arg[i],"r") 
	if f then
 	  table.insert(arg.files,arg[i]) -- insert filename into files table
	  f:close()
	else
          error("Command line: file "..arg[i].." does not exist")
	end            
      end
      i=i+1
   end
end

-------------------------------------------------------------------
-- assign all values in arg.values to global variables
function assignvalues(arg)
   table.foreach(arg.values,  
		 function(i,v) 
		    if _G[i] then 
		       print("ldf: overwritten: "..i.."="..v, "(was "..i.."=".._G[i]..")") 
		    else
		       print("ldf:   assigned: "..i.."="..v) 
		    end
		       _G[i]=v 
		    end
		 ) 
end





------------------------------------------------
---
--- Trail: Write-protection of namespaces
--- 

--[[
function swigext_protect(namespace)

 rawset(swigext_protected,namespace,true)
 
-- local tmp=_G[namespace]
-- rawset(_G,namespace,nil)
 
 local log=function (name,old,new)
  local t=debug.getinfo(3,"Sl")
  local line=t.currentline
  io.write(t.short_src)
  if line>=0 then io.write(":",line) end
  io.write(": ",name," is now ",tostring(new)," (was ",tostring(old),")","\n")
 end

 local err=function (name)
  local t=debug.getinfo(3,"Sl")
  local line=t.currentline
  io.write(t.short_src)
  if line>=0 then io.write(":",line) end
  print("namespace `"..name.."'".." is protected")
 end
 
 local g={}
 local set=function (t,name,value)
  --log(name,g[name],value)

  if swigext_protected[name] then
   log(name,g[name],value)
   err(name)
   --error("namespace `"..name.."'".." is protected")
  end

  --rawset(t,name,value)
  g[name]=value
 end
 
 if namespace == "grid" then
  --table.foreach(getfenv(),print)
 end
 
 setmetatable(getfenv(),{__index=g,__newindex=set})
-- _G[namespace]=tmp
end
]]

--[[
local function swigext_protect_index(t,i)
 print("t,i==",t,i)
 return rawget(t,i)
end
local function swigext_protect_newindex(t,i,v)
 print("t,i,v==",t,i,v)
 rawset(t,i,v)
end
--table.foreach(getfenv(),print)
setmetatable(getfenv(),{__index=swigext_protect_index,__newindex=swigext_protect_newindex})

function swigext_protect(namespace)
  rawset(swigext_protected,namespace,true)
end

]]

-- function swigext_protect(namespace) end
-- swigext_protected={}










using(mbase)


